@@ -95,9 +95,12 @@ gboolean no_locks= FALSE;
 gboolean dump_triggers= FALSE;
 gboolean dump_events= FALSE;
 gboolean dump_routines= FALSE;
+gboolean no_dump_views= FALSE;
 gboolean less_locking = FALSE;
 gboolean use_savepoints = FALSE;
 gboolean success_on_1146 = FALSE;
+gboolean no_backup_locks = FALSE;
+
 
 GList *innodb_tables= NULL;
 GList *non_innodb_table= NULL;
@@ -109,6 +112,7 @@ gint non_innodb_done= 0;
 guint less_locking_threads = 0;
 guint updated_since = 0;
 guint trx_consistency_only = 0;
+guint complete_insert = 0;
 
 // For daemon mode, 0 or 1
 guint dump_number= 0;
@@ -138,7 +142,9 @@ static GOptionEntry entries[] =
 	{ "triggers", 'G', 0, G_OPTION_ARG_NONE, &dump_triggers, "Dump triggers", NULL },
 	{ "events", 'E', 0, G_OPTION_ARG_NONE, &dump_events, "Dump events", NULL },
 	{ "routines", 'R', 0, G_OPTION_ARG_NONE, &dump_routines, "Dump stored procedures and functions", NULL },
+	{ "no-views", 'W', 0, G_OPTION_ARG_NONE, &no_dump_views, "Do not dump VIEWs", NULL },
 	{ "no-locks", 'k', 0, G_OPTION_ARG_NONE, &no_locks, "Do not execute the temporary shared read lock.  WARNING: This will cause inconsistent backups", NULL },
+	{ "no-backup-locks", 0, 0, G_OPTION_ARG_NONE, &no_backup_locks, "Do not use Percona backup locks", NULL},
 	{ "less-locking", 0, 0, G_OPTION_ARG_NONE, &less_locking, "Minimize locking time on InnoDB tables.", NULL},
 	{ "long-query-guard", 'l', 0, G_OPTION_ARG_INT, &longquery, "Set long query timer in seconds, default 60", NULL },
 	{ "kill-long-queries", 'K', 0, G_OPTION_ARG_NONE, &killqueries, "Kill long running queries (instead of aborting)", NULL },
@@ -155,6 +161,7 @@ static GOptionEntry entries[] =
 	{ "lock-all-tables", 0, 0, G_OPTION_ARG_NONE, &lock_all_tables, "Use LOCK TABLE for all, instead of FTWRL", NULL},
 	{ "updated-since", 'U', 0, G_OPTION_ARG_INT, &updated_since, "Use Update_time to dump only tables updated in the last U days", NULL},
 	{ "trx-consistency-only", 0, 0, G_OPTION_ARG_NONE, &trx_consistency_only, "Transactional consistency only", NULL},
+	{ "complete-insert", 0, 0, G_OPTION_ARG_NONE, &complete_insert, "Use complete INSERT statements that include column names", NULL},
 	{ NULL, 0, 0, G_OPTION_ARG_NONE,   NULL, NULL, NULL }
 };
 
@@ -332,7 +339,8 @@ void write_snapshot_info(MYSQL *conn, FILE *file) {
 			mastergtid=row[4];
 		} else {
 			/* Let's try with MariaDB 10.x */
-			mysql_query(conn, "SELECT @@gtid_current_pos");
+			/* Use gtid_binlog_pos due to issue with gtid_current_pos with galera cluster, gtid_binlog_pos works as well with normal mariadb server https://jira.mariadb.org/browse/MDEV-10279 */
+			mysql_query(conn, "SELECT @@gtid_binlog_pos");
 			mdb=mysql_store_result(conn);
 			if (mdb && (row=mysql_fetch_row(mdb))) {
 				mastergtid=row[0];
@@ -399,7 +407,10 @@ void *process_queue(struct thread_data *td) {
 	g_mutex_lock(init_mutex);
 	MYSQL *thrconn = mysql_init(NULL);
 	g_mutex_unlock(init_mutex);
-	
+
+	if (defaults_file != NULL) {
+		mysql_options(thrconn,MYSQL_READ_DEFAULT_FILE,defaults_file);
+	}
 	mysql_options(thrconn,MYSQL_READ_DEFAULT_GROUP,"mydumper");
 
 	if (compress_protocol)
@@ -420,11 +431,12 @@ void *process_queue(struct thread_data *td) {
 		g_warning("Failed to increase wait_timeout: %s", mysql_error(thrconn));
 	}
 	if (mysql_query(thrconn, "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ")) {
-		g_warning("Failed to set isolation level: %s", mysql_error(thrconn));
+		g_critical("Failed to set isolation level: %s", mysql_error(thrconn));
+		exit(EXIT_FAILURE);
 	}
 	if (mysql_query(thrconn, "START TRANSACTION /*!40108 WITH CONSISTENT SNAPSHOT */")) {
 		g_critical("Failed to start consistent snapshot: %s",mysql_error(thrconn));
-		errors++;
+		exit(EXIT_FAILURE);
 	}
 	if(!skip_tz && mysql_query(thrconn, "/*!40103 SET TIME_ZONE='+00:00' */")){
 		g_critical("Failed to set time zone: %s",mysql_error(thrconn));
@@ -599,7 +611,10 @@ void *process_queue_less_locking(struct thread_data *td) {
 	g_mutex_lock(init_mutex);
 	MYSQL *thrconn = mysql_init(NULL);
 	g_mutex_unlock(init_mutex);
-	
+
+	if (defaults_file != NULL) {
+		mysql_options(thrconn,MYSQL_READ_DEFAULT_FILE,defaults_file);
+	}
 	mysql_options(thrconn,MYSQL_READ_DEFAULT_GROUP,"mydumper");
 
 	if (compress_protocol)
@@ -937,6 +952,10 @@ MYSQL *create_main_connection()
 {
 	MYSQL *conn;
 	conn = mysql_init(NULL);
+
+	if (defaults_file != NULL) {
+		mysql_options(conn,MYSQL_READ_DEFAULT_FILE,defaults_file);
+	}
 	mysql_options(conn,MYSQL_READ_DEFAULT_GROUP,"mydumper");
 
 	if (!mysql_real_connect(conn, hostname, username, password, db, port, socket_path, 0)) {
@@ -1005,6 +1024,9 @@ void *binlog_thread(void *data) {
 	MYSQL_ROW row;
 	MYSQL *conn;
 	conn = mysql_init(NULL);
+	if (defaults_file != NULL) {
+		mysql_options(thrconn,MYSQL_READ_DEFAULT_FILE,defaults_file);
+	}
 	mysql_options(conn,MYSQL_READ_DEFAULT_GROUP,"mydumper");
 
 	if (!mysql_real_connect(conn, hostname, username, password, db, port, socket_path, 0)) {
@@ -1052,6 +1074,7 @@ void start_dump(MYSQL *conn)
 	struct schema_post *sp;
 	guint n;
 	FILE* nufile = NULL;
+	guint have_backup_locks = 0;
 	
 	for(n=0;n<num_threads;n++){
 		nits[n] = 0;
@@ -1128,7 +1151,28 @@ void start_dump(MYSQL *conn)
 	}
 
 	if (!no_locks) {
-		if(lock_all_tables){
+		// Percona Backup Locks
+		if(!no_backup_locks){
+			mysql_query(conn,"SELECT @@have_backup_locks");
+			MYSQL_RES *rest = mysql_store_result(conn);
+			if(rest != NULL && mysql_num_rows(rest)){
+				mysql_free_result(rest);
+				g_message("Using Percona Backup Locks");
+				have_backup_locks=1;
+			}
+		}
+
+		if(have_backup_locks){
+			if(mysql_query(conn, "LOCK TABLES FOR BACKUP")) {
+				g_critical("Couldn't acquire LOCK TABLES FOR BACKUP, snapshots will not be consistent: %s",mysql_error(conn));
+				errors++;
+			}
+
+			if(mysql_query(conn, "LOCK BINLOG FOR BACKUP")) {
+				g_critical("Couldn't acquire LOCK BINLOG FOR BACKUP, snapshots will not be consistent: %s",mysql_error(conn));
+				errors++;
+			}
+		}else if(lock_all_tables){
 			// LOCK ALL TABLES
 			GString *query= g_string_sized_new(16777216);
 			gchar *dbtb = NULL;
@@ -1303,6 +1347,8 @@ void start_dump(MYSQL *conn)
 	if (trx_consistency_only){
 		g_message("Transactions started, unlocking tables");
 		mysql_query(conn, "UNLOCK TABLES /* trx-only */");
+		if(have_backup_locks)
+			mysql_query(conn, "UNLOCK BINLOG");
 	}
 
 	if (db) {
@@ -1416,6 +1462,8 @@ void start_dump(MYSQL *conn)
 		g_async_queue_pop(conf.unlock_tables);
 		g_message("Non-InnoDB dump complete, unlocking tables");
 		mysql_query(conn, "UNLOCK TABLES /* FTWRL */");
+		if(have_backup_locks)
+			mysql_query(conn, "UNLOCK BINLOG");
 	}
 	#ifdef WITH_BINLOG
 	if (need_binlogs) {
@@ -1602,6 +1650,11 @@ GList * get_chunks_for_table(MYSQL *conn, char *database, char *table, struct co
 
 	row=mysql_fetch_row(minmax);
 	MYSQL_FIELD * fields=mysql_fetch_fields(minmax);
+
+	/* Check if all values are NULL */
+	if (row[0] == NULL)
+		goto cleanup;
+
 	char *min=row[0];
 	char *max=row[1];
 
@@ -1789,6 +1842,11 @@ void dump_database(MYSQL * conn, char *database, FILE *file, struct configuratio
 				}
 			}
 		}
+
+		/* Skip views */
+		if (is_view && no_dump_views)
+			dump = 0;
+
 		if (!dump)
 			continue;
 
@@ -2710,7 +2768,18 @@ guint64 dump_table_data(MYSQL * conn, FILE *file, char *database, char *table, c
 					return num_rows;
 				}
 			}
-			g_string_printf(statement, "INSERT INTO `%s` VALUES", table);
+			if (complete_insert) {
+				g_string_printf(statement, "INSERT INTO `%s` (", table);
+				for (i = 0; i < num_fields; ++i) {
+					if (i > 0) {
+						g_string_append_c(statement, ',');
+					}
+					g_string_append_printf(statement, "`%s`", fields[i].name);
+				}
+				g_string_append(statement, ") VALUES");
+			} else {
+				g_string_printf(statement, "INSERT INTO `%s` VALUES", table);
+			}
 			num_rows_st = 0;
 		}
 		
@@ -2790,7 +2859,18 @@ guint64 dump_table_data(MYSQL * conn, FILE *file, char *database, char *table, c
 			g_string_append(statement, statement_row->str);
 		}
 		else {
-			g_string_printf(statement, "INSERT INTO `%s` VALUES", table);
+			if (complete_insert) {
+				g_string_printf(statement, "INSERT INTO `%s` (", table);
+				for (i = 0; i < num_fields; ++i) {
+					if (i > 0) {
+						g_string_append_c(statement, ',');
+					}
+					g_string_append_printf(statement, "`%s`", fields[i].name);
+				}
+				g_string_append(statement, ") VALUES");
+			} else {
+				g_string_printf(statement, "INSERT INTO `%s` VALUES", table);
+			}
 			g_string_append(statement, statement_row->str);
 		}
 	}
